/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.log;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * {@link NullSafeConcurrentHashMap}
 * 
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class NullSafeConcurrentHashMap<K, V> extends ConcurrentHashMap<K, V> {

    private static final long serialVersionUID = 6289778447816506246L;

    private final ConcurrentHashMap<K, V> delegatee;

    /**
     * Initializes a new {@link NullSafeConcurrentHashMap}.
     */
    protected NullSafeConcurrentHashMap() {
        super(1);
        delegatee = new ConcurrentHashMap<K, V>();
    }

    /**
     * Initializes a new {@link NullSafeConcurrentHashMap}.
     * 
     * @param initialCapacity The initial capacity. The implementation performs internal sizing to accommodate this many elements.
     * @param loadFactor The load factor threshold, used to control resizing. Resizing may be performed when the average number of elements per bin exceeds this threshold.
     * @param concurrencyLevel The estimated number of concurrently updating threads. The implementation performs internal sizing to try to accommodate this many threads.
     */
    protected NullSafeConcurrentHashMap(final int initialCapacity, final float loadFactor, final int concurrencyLevel) {
        super(1);
        delegatee = new ConcurrentHashMap<K, V>(initialCapacity, loadFactor, concurrencyLevel);
    }

    /**
     * Initializes a new {@link NullSafeConcurrentHashMap}.
     * 
     * @param initialCapacity The initial capacity. The implementation performs internal sizing to accommodate this many elements.
     * @param loadFactor The load factor threshold, used to control resizing. Resizing may be performed when the average number of elements per bin exceeds this threshold.
     */
    protected NullSafeConcurrentHashMap(final int initialCapacity, final float loadFactor) {
        super(1);
        delegatee = new ConcurrentHashMap<K, V>(initialCapacity, loadFactor);
    }

    /**
     * Initializes a new {@link NullSafeConcurrentHashMap}.
     * 
     * @param initialCapacity The initial capacity. The implementation performs internal sizing to accommodate this many elements.
     */
    protected NullSafeConcurrentHashMap(final int initialCapacity) {
        super(1);
        delegatee = new ConcurrentHashMap<K, V>(initialCapacity);
    }

    /**
     * Initializes a new {@link NullSafeConcurrentHashMap}.
     * 
     * @param m The map
     */
    protected NullSafeConcurrentHashMap(final Map<? extends K, ? extends V> m) {
        super(1);
        delegatee = new ConcurrentHashMap<K, V>(m);
    }

    @Override
    public boolean equals(final Object o) {
        return delegatee.equals(o);
    }

    @Override
    public int hashCode() {
        return delegatee.hashCode();
    }

    @Override
    public String toString() {
        return delegatee.toString();
    }

    @Override
    public boolean isEmpty() {
        return delegatee.isEmpty();
    }

    @Override
    public int size() {
        return delegatee.size();
    }

    @Override
    public V get(final Object key) {
        return delegatee.get(key);
    }

    @Override
    public boolean containsKey(final Object key) {
        return delegatee.containsKey(key);
    }

    @Override
    public boolean containsValue(final Object value) {
        return delegatee.containsValue(value);
    }

    @Override
    public boolean contains(final Object value) {
        return delegatee.contains(value);
    }

    @Override
    public V put(final K key, final V value) {
        if (null == value) {
            return delegatee.remove(key);
        }
        return delegatee.put(key, value);
    }

    @Override
    public V putIfAbsent(final K key, final V value) {
        return delegatee.putIfAbsent(key, value);
    }

    @Override
    public void putAll(final Map<? extends K, ? extends V> m) {
        delegatee.putAll(m);
    }

    @Override
    public V remove(final Object key) {
        return delegatee.remove(key);
    }

    @Override
    public boolean remove(final Object key, final Object value) {
        return delegatee.remove(key, value);
    }

    @Override
    public boolean replace(final K key, final V oldValue, final V newValue) {
        return delegatee.replace(key, oldValue, newValue);
    }

    @Override
    public V replace(final K key, final V value) {
        return delegatee.replace(key, value);
    }

    @Override
    public void clear() {
        delegatee.clear();
    }

    @Override
    public Set<K> keySet() {
        return delegatee.keySet();
    }

    @Override
    public Collection<V> values() {
        return delegatee.values();
    }

    @Override
    public Set<java.util.Map.Entry<K, V>> entrySet() {
        return delegatee.entrySet();
    }

    @Override
    public Enumeration<K> keys() {
        return delegatee.keys();
    }

    @Override
    public Enumeration<V> elements() {
        return delegatee.elements();
    }

}
